from gevent import monkey; monkey.patch_all()

from xaal.lib import Engine,Device,helpers,tools
from bottle import default_app,debug,get,request,response,redirect,static_file,template,TEMPLATE_PATH

from . import devices
import os
import platform
import logging

from gevent import Greenlet
from gevent.pywsgi import WSGIServer
from geventwebsocket.handler import WebSocketHandler

PACKAGE_NAME = "xaal.fakeinput"
logger = logging.getLogger(PACKAGE_NAME)

DEFAULT_PORT = 8081
HOME = os.path.dirname(__file__)


cfg = None
fake_devices = []

def setup_xaal():
    """ setup xAAL Engine & Device. And start it in a Greenlet"""
    global monitor,cfg
    engine = Engine()
    cfg = tools.load_cfg(PACKAGE_NAME)
    if not cfg:
        cfg = tools.new_cfg(PACKAGE_NAME)
        logger.info(f"No config file found, building a new one {cfg.filename}")
        cfg['config']['port'] = DEFAULT_PORT
        cfg['devices'] = random_devices()
        cfg.write()
    dev            = Device("hmi.basic")
    dev.address    = tools.get_uuid(cfg['config']['addr'])
    dev.vendor_id  = "IHSEV"
    dev.product_id = "Fake buttons, switches, contact"
    dev.version    = 0.1
    dev.info       = "%s@%s" % (PACKAGE_NAME,platform.node())

    engine.add_device(dev)
    load_config(engine)

    engine.start()        
    green_let = Greenlet(xaal_loop, engine)
    green_let.start()

def xaal_loop(engine):
    """ xAAL Engine Loop Greenlet"""
    while 1:
        engine.loop()

def random_devices():
    tmp = {}
    for k in ['button','contact','switch']:
        uid = str(tools.get_random_uuid())
        tmp[uid] = {'name':'fake %s' % k,'type':k}
    return tmp

def load_config(engine):
    global fake_devices
    cfg_devices = cfg.get('devices',[])
    for k in cfg_devices:
        addr = tools.get_uuid(k)
        type_ = cfg_devices[k].get('type',None)
        name =  cfg_devices[k].get('name',None)
        dev = None
        if type_ :
            if type_ == 'button':
                dev = devices.Button(addr,name,engine)
            if type_ == 'switch':
                dev = devices.Switch(addr,name,engine)
            if type_ == 'contact':
                dev = devices.Contact(addr,name,engine)
        if dev:
            fake_devices.append(dev)

def search_device(addr):
    uid = tools.str_to_uuid(addr)
    for k in fake_devices:
        if k.dev.address == uid:
            return k
    return None

@get('/static/<filename:path>')
def static(filename):
    root = os.path.dirname(__file__)
    root = os.path.join(root,'static')
    return static_file(filename, root=root)

@get('/')
def index():
    return template('index.tpl',request=request,devices = fake_devices) 

@get('/api/click/<addr>/<click_type:int>')
def click(addr,click_type):
    dev = search_device(addr)
    if dev and isinstance(dev,devices.Button):
        dev.click(click_type)

@get('/api/set_on/<addr>')
def set_on(addr):
    dev = search_device(addr)
    if dev and not isinstance(dev,devices.Button):
        dev.set_on()

@get('/api/set_off/<addr>')
def set_off(addr):
    dev = search_device(addr)
    if dev and not isinstance(dev,devices.Button):
        dev.set_off()


def run():
    """ start the xAAL stack & launch the HTTP stuff"""
    root = os.path.join(HOME,'views')
    TEMPLATE_PATH.append(root)
    helpers.set_console_title(PACKAGE_NAME)
    helpers.setup_console_logger(level=logging.INFO)
    setup_xaal()
    app = default_app()
    debug(True)
    port = int(cfg.get('config').get('port',DEFAULT_PORT))
    logger.info("HTTP Server running on port : %d" % port)
    server = WSGIServer(("", port), app, handler_class=WebSocketHandler)
    server.serve_forever()

def main():
    try:
        run()
    except KeyboardInterrupt:
        print("Bye Bye...")
    
if __name__ == '__main__':
    main()
